from marshmallow import Schema, fields, post_load, validates, ValidationError, validates_schema
from app.users.models import User
import re
from werkzeug.security import check_password_hash
from flask import request

class UserSchema(Schema):
    first_name = fields.String(required=True)
    last_name = fields.String(required=True)
    role_id = fields.String(required=True)
    customer_id = fields.Integer(required=False)
    site_id = fields.Integer(required=True)
    zone_id = fields.Str(required=True)
    room_id = fields.Integer(required=True)
    email = fields.String(required=True)
    password = fields.Str(required=True)
    confirm_password = fields.Str(required=True)
    device_name = fields.String(required=True)
    device_id = fields.String(required=True)
    preference = fields.String(required=True)
    logo_id = fields.Integer(required=False)
    @validates("first_name")
    def required_first_name(self, value):
        if not value:
            raise ValidationError("The First Name field is required")
    @validates("role_id")
    def required_role_id(self, value):
        if not value:
            raise ValidationError("The Role Type field is required")

    @validates("confirm_password")
    def required_confirm_password(self, value):
        if request.method != 'PUT' or value:
            if not value:
                raise ValidationError("The confirm Password field is required")
    @validates_schema()
    def validate_password(self, value, **kwargs):
        if request.method != 'PUT' or value:
            if value['password'] != value['confirm_password']:
                raise ValidationError("The password and confirmation password do not match")
    @validates("password")
    def validates_password(self, value):
        if request.method != 'PUT' or value:
            if not value:
                raise ValidationError("The Password field is required")
            if len(value) < 6:
                raise ValidationError("Password must longer than 6")
    @validates("email")
    def validates_email(self, value):
        if not value:
            raise ValidationError("The Email field is required")
        if not re.match("[^@]+@[^@]+\.[^@]+", value):
            raise ValidationError("Invalid email format")
        if request.method == 'POST':
            existing_user = User.query.filter_by(email=value).first()
            if existing_user:
                raise ValidationError("Email already exists")
    @post_load
    def make_user(self, data, **kwargs):
        return data
user_schema = UserSchema()
users_schema = UserSchema(many=True)

class PasswordSchema(Schema):
    password=fields.Str(required=True)
    confirm_password = fields.Str(required=True)
    @validates("confirm_password")
    def required_confirm_password(self, value):
        if not value:
            raise ValidationError("The confirm Password field is required")
    @validates_schema()
    def validate_password(self, value, **kwargs):
        if value['password'] != value['confirm_password']:
            raise ValidationError("The password and confirmation password do not match")
    @validates("password")
    def validates_password(self, value):
        if not value:
            raise ValidationError("The Password field is required")
        if len(value) < 6:
            raise ValidationError("Password must longer than 6")
    @post_load
    def make_user(self, data, **kwargs):
        return data
password_schema = PasswordSchema()
password_schemas = PasswordSchema(many=True)

class LoginSchema(Schema):
    email = fields.String(required=True)
    password = fields.String(required=True)

    @validates("password")
    def validates_password(self, value):
        if not value:
            raise ValidationError("The Password field is required")
        if len(value) < 6:
            raise ValidationError("Password must longer than 6")
        existing_user = User.query.filter_by(password=value).first()
        if existing_user and not check_password_hash(existing_user.password, self.password):
            raise ValidationError("Invalid Credentials")
    @validates("email")
    def validates_email(self, value):
        if not value:
            raise ValidationError("The Email field is required")
        existing_user = User.query.filter_by(email=value).first()
        if not existing_user:
            raise ValidationError(" Please enter valid User. User not registered")
        if not re.match("[^@]+@[^@]+\.[^@]+", value):
            raise ValidationError("Invalid email format")
    @post_load
    def make_user(self, data, **kwargs):
        return data
login_schema = LoginSchema()
login_schemas = LoginSchema(many=True)

class UserUpdateSchema(Schema):
    first_name = fields.String(required=True)
    last_name = fields.String(required=True)
    role_id = fields.Integer(required=True)
    email = fields.String(required=True)
    @validates("first_name")
    def requiredfirst_name(self, value):
        if not value:
            raise ValidationError("The First Name field is required")
    @validates("role_id")
    def requiredrole_id(self, value):
        if not value:
            raise ValidationError("The role id field is required")
    @validates("last_name")
    def requiredlast_name(self, value):
        if not value:
            raise ValidationError("The Last Name field is required")
        
    def make_user(self, data, **kwargs):
        return data
user_update_schema = UserUpdateSchema()
users_update_schemas = UserUpdateSchema(many=True)
